@@ -26,7 +26,7 @@
 #include <string.h>
 #include <time.h>
 
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #include "sierra.h"
 #include "sierra-usbwrap.h"
@@ -67,6 +67,9 @@ enum _SierraPacket {
 	SIERRA_PACKET_SESSION_END	= 0xff
 };
 
+/* Size of requested packet */
+#define SIERRA_PACKET_SIZE		32774
+
 /* Sub-types */
 #define SUBSIERRA_PACKET_COMMAND_FIRST   0x53
 #define SUBSIERRA_PACKET_COMMAND         0x43
@@ -444,8 +447,10 @@ sierra_write_packet (Camera *camera, char *packet, GPContext *context)
 		packet[length-1] = (checksum >> 8) & 0xff; 
 	}
 
-	if (camera->pl->flags & SIERRA_WRAP_USB) {
-		CHECK (usb_wrap_write_packet (camera->port, packet, length));
+	if (camera->pl->flags & SIERRA_WRAP_USB_MASK) {
+		CHECK (usb_wrap_write_packet (camera->port,
+			(camera->pl->flags & SIERRA_WRAP_USB_MASK),
+			packet, length));
 	} else {
 		CHECK (gp_port_write (camera->port, packet, length));
 	}
@@ -453,6 +458,17 @@ sierra_write_packet (Camera *camera, char *packet, GPContext *context)
 	return GP_OK;
 }
 
+static int
+sierra_clear_usb_halt(Camera *camera)
+{
+	
+	if ( (camera->port->type == GP_PORT_USB) &&
+	     !(camera->pl->flags & SIERRA_WRAP_USB_MASK) &&
+	     !(camera->pl->flags & SIERRA_NO_USB_CLEAR) )
+		gp_port_usb_clear_halt(camera->port, GP_PORT_USB_ENDPOINT_IN);
+        return (GP_OK);
+}
+
 static int
 sierra_write_nak (Camera *camera, GPContext *context)
 {
@@ -463,8 +479,7 @@ sierra_write_nak (Camera *camera, GPContext *context)
 
         buf[0] = SIERRA_PACKET_NAK;
         ret = sierra_write_packet (camera, buf, context);
-        if (camera->port->type == GP_PORT_USB && !(camera->pl->flags & SIERRA_WRAP_USB))
-                gp_port_usb_clear_halt(camera->port, GP_PORT_USB_ENDPOINT_IN);
+	sierra_clear_usb_halt(camera);
         return (ret);
 }
 
@@ -508,7 +523,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 
 	switch (camera->port->type) {
 	case GP_PORT_USB:
-		blocksize = 32774;
+		blocksize = SIERRA_PACKET_SIZE;
 		break;
 	case GP_PORT_SERIAL:
 		blocksize = 1;
@@ -522,27 +537,23 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 
 		/* Clear the USB bus
 		   (what about the SERIAL bus : do we need to flush it?) */
-		if (camera->port->type == GP_PORT_USB && !(camera->pl->flags & SIERRA_WRAP_USB))
-			gp_port_usb_clear_halt (camera->port,
-						GP_PORT_USB_ENDPOINT_IN);
+		sierra_clear_usb_halt(camera);
 
 		/*
 		 * Read data through the bus. If an error occurred,
 		 * try again.
 		 */
-		if (camera->port->type == GP_PORT_USB && (camera->pl->flags & SIERRA_WRAP_USB))
-			result = usb_wrap_read_packet (camera->port, packet,
-						       blocksize);
+		if (camera->port->type == GP_PORT_USB && (camera->pl->flags & SIERRA_WRAP_USB_MASK))
+			result = usb_wrap_read_packet (camera->port,
+					(camera->pl->flags & SIERRA_WRAP_USB_MASK),
+					packet, blocksize);
 		else
 			result = gp_port_read (camera->port, packet, blocksize);
 		if (result < 0) {
 			GP_DEBUG ("Read failed (%i: '%s').", result,
 				  gp_result_as_string (result));
 			if (++r > 2) {
-				if (camera->port->type == GP_PORT_USB &&
-				    !(camera->pl->flags & SIERRA_WRAP_USB))
-					gp_port_usb_clear_halt (camera->port,
-						GP_PORT_USB_ENDPOINT_IN);
+				sierra_clear_usb_halt(camera);
 				GP_DEBUG ("Giving up...");
 				return (result);
 			}
@@ -566,11 +577,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 		case SIERRA_PACKET_WRONG_SPEED:
 
 			/* Those are all single byte packets. */
-			if (camera->port->type == GP_PORT_USB &&
-			    !(camera->pl->flags & SIERRA_WRAP_USB))
-				gp_port_usb_clear_halt (camera->port,
-						GP_PORT_USB_ENDPOINT_IN);
-
+			sierra_clear_usb_halt(camera);
 			GP_DEBUG ("Packet read. Returning GP_OK.");
 			return GP_OK;
 
@@ -591,10 +598,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 				"received (0x%x) is not valid."), packet[0]);
 			while (gp_port_read (camera->port, packet, 1) > 0)
 				;
-			if (camera->port->type == GP_PORT_USB &&
-			    !(camera->pl->flags & SIERRA_WRAP_USB))
-				gp_port_usb_clear_halt (camera->port,
-						GP_PORT_USB_ENDPOINT_IN);
+			sierra_clear_usb_halt(camera);
 			return (GP_ERROR_CORRUPTED_DATA);
 		}
 
@@ -607,10 +611,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 			result = gp_port_read (camera->port,
 					       packet + br, 4 - br);
 			if (result < 0) {
-				if (camera->port->type == GP_PORT_USB &&
-				    !(camera->pl->flags & SIERRA_WRAP_USB))
-					gp_port_usb_clear_halt (camera->port,
-						GP_PORT_USB_ENDPOINT_IN);
+				sierra_clear_usb_halt(camera);
 				GP_DEBUG ("Could not read length of "
 					"packet (%i: '%s'). Giving up...",
 					result, gp_result_as_string (result));
@@ -628,7 +629,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 
 		/* 
 		 * Read until the end of the packet is reached
-		 * or an error occured.
+		 * or an error occurred.
 		 */
 		while (br < length) {
 			result = gp_port_read (camera->port, packet + br,
@@ -668,7 +669,15 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 			if ((packet[br - 2] == 0xff) &&
 			    (packet[br - 1] == 0xff))
 				break;
-
+			/*
+			 * The checksum for the Toshiba PDR-M6X cameras is
+			 * almost always set to 0x00 0x00.
+			 */
+			if ((packet[br - 2] == 0x00) &&
+			    (packet[br - 1] == 0x00))
+				break;
+			
+					
 			GP_DEBUG ("Checksum wrong (calculated 0x%x, "
 				"found 0x%x)!", c,
 				packet[br - 2] + (packet[br - 1] * 256));
@@ -680,10 +689,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 		 */
 		r++;
 		if (r + 1 >= RETRIES) {
-			if (camera->port->type == GP_PORT_USB &&
-			    !(camera->pl->flags & SIERRA_WRAP_USB))
-				gp_port_usb_clear_halt (camera->port,
-						GP_PORT_USB_ENDPOINT_IN);
+			sierra_clear_usb_halt(camera);
 			GP_DEBUG ("Giving up...");
 			return ((br == length) ? GP_ERROR_CORRUPTED_DATA :
 						 GP_ERROR_TIMEOUT);
@@ -693,8 +699,7 @@ sierra_read_packet (Camera *camera, unsigned char *packet, GPContext *context)
 		GP_SYSTEM_SLEEP (10);
 	}
 
-	if (camera->port->type == GP_PORT_USB && !(camera->pl->flags & SIERRA_WRAP_USB))
-		gp_port_usb_clear_halt(camera->port, GP_PORT_USB_ENDPOINT_IN);
+	sierra_clear_usb_halt(camera);
 
 	return GP_OK;
 }
@@ -733,7 +738,7 @@ static int
 sierra_transmit_ack (Camera *camera, char *packet, GPContext *context)
 {
 	int r = 0, result;
-	unsigned char buf[4096];
+	unsigned char buf[SIERRA_PACKET_SIZE];
 
 	while (1) {
 		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL)
@@ -835,8 +840,7 @@ sierra_write_ack (Camera *camera, GPContext *context)
 	
 	buf[0] = ACK;
 	ret = sierra_write_packet (camera, buf, context);
-	if (camera->port->type == GP_PORT_USB && !(camera->pl->flags & SIERRA_WRAP_USB))
-		gp_port_usb_clear_halt (camera->port, GP_PORT_USB_ENDPOINT_IN);
+	sierra_clear_usb_halt(camera);
 	CHECK (ret);
 
 	GP_DEBUG ("Successfully wrote acknowledgement.");
@@ -846,7 +850,7 @@ sierra_write_ack (Camera *camera, GPContext *context)
 int
 sierra_init (Camera *camera, GPContext *context) 
 {
-	unsigned char buf[4096], packet[4096];
+	unsigned char buf[SIERRA_PACKET_SIZE], packet[4096];
 	int ret, r = 0;
 	GPPortSettings settings;
 
@@ -979,7 +983,7 @@ sierra_set_speed (Camera *camera, SierraSpeed speed, GPContext *context)
 int sierra_sub_action (Camera *camera, SierraAction action, int sub_action,
 		       GPContext *context)
 {
-	char buf[4096];
+	char buf[SIERRA_PACKET_SIZE];
 
 	CHECK (sierra_build_packet (camera, SIERRA_PACKET_COMMAND, 0, 3, buf));
 	buf[4] = 0x02;
@@ -1040,7 +1044,7 @@ sierra_set_int_register (Camera *camera, int reg, int value,
 int sierra_get_int_register (Camera *camera, int reg, int *value, GPContext *context) 
 {
 	int r = 0;
-	unsigned char p[4096], buf[4096];
+	unsigned char p[4096], buf[SIERRA_PACKET_SIZE];
 
 	GP_DEBUG ("sierra_get_int_register: register 0x%02x...", reg);
 
@@ -1120,14 +1124,14 @@ sierra_set_string_register (Camera *camera, int reg, const char *s,
 	char packet[4096];
 	char type;
 	long int x=0;
-	int seq=0, size=0, do_percent;
+	int seq=0, size=0; 
+	int do_percent;
 	unsigned int id = 0;
 
 	GP_DEBUG ("sierra_set_string_register: reg %i, value '%s'", reg, s);
 
 	/* Make use of the progress bar when the packet is "large enough" */
 	if (length > MAX_DATA_FIELD_LENGTH) {
-#ifdef RECURSION_REMOVED
 		do_percent = 1;
 		/*
 		 * This code can lead to recursive calls in gtkam - the
@@ -1139,7 +1143,6 @@ sierra_set_string_register (Camera *camera, int reg, const char *s,
 		 */
 		id = gp_context_progress_start (context, length,
 						_("Sending data..."));
-#endif
 	}
 	else
 		do_percent = 0;
@@ -1172,10 +1175,8 @@ sierra_set_string_register (Camera *camera, int reg, const char *s,
 
 		/* Transmit packet */
 		CHECK (sierra_transmit_ack (camera, packet, context));
-#ifdef RECURSION_REMOVED
 		if (do_percent)
 			gp_context_progress_update (context, id, x);
-#endif
 	}
 	if (do_percent)
 		gp_context_progress_stop (context, id);
@@ -1188,11 +1189,12 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 				unsigned int *b_len, GPContext *context)
 {
 	unsigned char p[34816];
-	const char *file_name;
 	unsigned int packlength, total = (b_len) ? *b_len : 0;
-	unsigned int id = 0;
 	int retries, r;
+	unsigned int min_progress_bytes;
 	static int in_function = 0;
+	const char *file_name;
+	unsigned int id = 0;
 
 	GP_DEBUG ("sierra_get_string_register:  reg %i, file number %i, "
 		  " total %d, flags 0x%x", reg, fnumber, total,
@@ -1217,13 +1219,19 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 	 * and receive 32k size packages, otherwise code 0x04 is used and
 	 * we have 2k size packages.
 	 */
-	p[4] = (camera->pl->flags & SIERRA_EXT_PROTO) ? 0x06 : 0x04;
+	if (camera->pl->flags & SIERRA_EXT_PROTO) {
+		p[4] = 0x06;
+		min_progress_bytes = 32 * 1024;
+	} else {
+		p[4] = 0x04;
+		min_progress_bytes = 2 * 1024;
+	}
 	p[5] = reg;
 	CHECK (sierra_write_packet (camera, p, context));
 
-	if (file && total) {
+	if (file && total > min_progress_bytes) {
 		CHECK (gp_file_get_name(file, &file_name));
-		id = gp_context_progress_start (context, total, file_name);
+		id = gp_context_progress_start (context, total, "%s", file_name);
 	}
 
 	/* Read all the data packets */
@@ -1267,14 +1275,12 @@ int sierra_get_string_register (Camera *camera, int reg, int fnumber,
 
 		if (file) {
 		    CHECK (gp_file_append (file, &p[4], packlength));
-#ifdef GTKAM_IS_MODIFIED
-		    if (total)
+		    if (total > min_progress_bytes)
 			gp_context_progress_update (context, id, *b_len);
-#endif
 		}
 
 	} while (p[0] != SIERRA_PACKET_DATA_END);
-	if (file && total)
+	if (file && total > min_progress_bytes)
 		gp_context_progress_stop (context, id);
 
 	GP_DEBUG ("sierra_get_string_register: completed OK, *b_len %d",
