@@ -26,8 +26,8 @@
 #include <string.h>
 #include <time.h>
 
-#include <gphoto2-library.h>
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #include "sierra-desc.h"
 #include "library.h"
@@ -56,10 +56,10 @@
 int get_jpeg_data(const char *data, int data_size, char **jpeg_data, int *jpeg_size);
 
 /* Useful markers */
-const char JPEG_SOI_MARKER[]  = { (char)0xFF, (char)0xD8, '\0' };
-const char JPEG_SOF_MARKER[]  = { (char)0xFF, (char)0xD9, '\0' };
-const char JPEG_APP1_MARKER[] = { (char)0xFF, (char)0xE1, '\0' };
-const char TIFF_SOI_MARKER[]  = { (char)0x49, (char)0x49, (char)0x2A, (char)0x00, (char)0x08, '\0' };
+static const char JPEG_SOI_MARKER[]  = { (char)0xFF, (char)0xD8, '\0' };
+static const char JPEG_SOF_MARKER[]  = { (char)0xFF, (char)0xD9, '\0' };
+static const char JPEG_APP1_MARKER[] = { (char)0xFF, (char)0xE1, '\0' };
+static const char TIFF_SOI_MARKER[]  = { (char)0x49, (char)0x49, (char)0x2A, (char)0x00, (char)0x08, '\0' };
 
 static struct {
 	const char *manuf;
@@ -84,8 +84,9 @@ static struct {
 	{"Epson", "PhotoPC 500", SIERRA_MODEL_EPSON,	0, 0, 0 },
 	{"Epson", "PhotoPC 550", SIERRA_MODEL_EPSON,	0, 0, 0 },
 	{"Epson", "PhotoPC 600", SIERRA_MODEL_EPSON,	0, 0, 0 },
-	{"Epson", "PhotoPC 650", SIERRA_MODEL_EPSON,	0, 0, 0 },
-	{"Epson", "PhotoPC 700", SIERRA_MODEL_EPSON,	0, 0, 0 },
+	{"Epson", "PhotoPC 650", SIERRA_MODEL_EPSON,	0, 0,
+					SIERRA_NO_51 | SIERRA_LOW_SPEED },
+	{"Epson", "PhotoPC 700", SIERRA_MODEL_EPSON,	0, 0, SIERRA_NO_51 },
 	{"Epson", "PhotoPC 800", SIERRA_MODEL_EPSON,	0, 0, 0 },
 	{"Epson", "PhotoPC 850z", SIERRA_MODEL_EPSON,     0x4b8, 0x402, 0},
 	{"Epson", "PhotoPC 3000z", SIERRA_MODEL_CAM_DESC,	0x4b8, 0x403, 0, &ep3000z_cam_desc},
@@ -102,8 +103,26 @@ static struct {
 	{"Nikon", "CoolPix 950S", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
 	{"Nikon", "CoolPix 990", SIERRA_MODEL_DEFAULT,	0x04b0, 0x0102, 0},
 	{"Nikon", "CoolPix 995",   SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0104,
-						SIERRA_WRAP_USB,
+						SIERRA_WRAP_USB_OLYMPUS,
 							&cp995_cam_desc},
+
+	{"Nikon", "CoolPix 2500 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0108,
+						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
+							&cp2500_cam_desc},
+
+	{"Nikon", "CoolPix 2100 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0116,
+						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
+							&cp2500_cam_desc},
+
+	{"Nikon", "CoolPix 4300 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x010e,
+						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
+							&cp880_cam_desc},
+	{"Nikon", "CoolPix 3500 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0110,
+						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
+							&cp880_cam_desc},
+	{"Nikon", "D100 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0401,
+						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
+							&cp880_cam_desc},
 	{"Olympus", "D-100Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "D-200L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "D-220L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
@@ -125,25 +144,35 @@ static struct {
 	{"Olympus", "C-410", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-410L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-420", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus", "C-420L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-420L", 	SIERRA_MODEL_OLYMPUS,	0, 0, SIERRA_MID_SPEED },
 	{"Olympus", "C-700UZ",	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+	{"Olympus", "C-750UZ",	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
+					SIERRA_WRAP_USB_OLYMPUS, &oly750uz_cam_desc},
+	{"Olympus", "C-770UZ",	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
+					SIERRA_WRAP_USB_OLYMPUS, &oly750uz_cam_desc},
 	{"Olympus", "C-800", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-800L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-820", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus", "C-820L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-820L", 	SIERRA_MODEL_OLYMPUS,	0, 0,
+					SIERRA_MID_SPEED },
 	{"Olympus", "C-830L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-840L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-860L",	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-900 Zoom", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-900L Zoom", SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+
+	/* http://sourceforge.net/tracker/index.php?func=detail&aid=1082569&group_id=8874&atid=358874 
+	 * strangely only works with low speed
+	 */
+	{"Olympus", "C-990 Zoom", 	SIERRA_MODEL_OLYMPUS,	0, 0, SIERRA_LOW_SPEED },
 	{"Olympus", "C-1000L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-1400L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-1400XL", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-2000Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-2020Z",	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-2040Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-2100UZ", SIERRA_MODEL_CAM_DESC, 0x07b4, 0x100, 0,
 					&oly3040_cam_desc},
 	{"Olympus", "C-2500L",     SIERRA_MODEL_OLYMPUS,   0, 0, 0 },
@@ -156,16 +185,27 @@ static struct {
 	{"Olympus", "C-3000Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x100, 0,
 							&oly3000z_cam_desc},
 	{"Olympus", "C-3020Z", SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-3030Z", SIERRA_MODEL_CAM_DESC,	0x07b4, 0x100, 0,
 							&oly3040_cam_desc},
 	{"Olympus", "C-3040Z",     SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-4040Z",     SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-5050Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+	{"Olympus", "C-370Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x109,
+				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+	{"Olympus", "X-450", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x109,
+				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+	{"Olympus", "D-535Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x109,
+				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+
 	{"Panasonic", "Coolshot NV-DCF5E", SIERRA_MODEL_DEFAULT, 0, 0, 0 },
+
+	{"Pentax", "Optio 450", SIERRA_MODEL_DEFAULT, 0x0a17,0x0007, SIERRA_WRAP_USB_PENTAX },
+	{"Pentax", "Optio 33WR", SIERRA_MODEL_DEFAULT, 0x0a17,0x0009, SIERRA_WRAP_USB_PENTAX },
+
 	{"Polaroid", "PDC 640", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
 	{"Polaroid", "PDC 2300Z", SIERRA_MODEL_DEFAULT, 0x0546, 0x0daf,
 					SIERRA_SKIP_INIT },
@@ -174,8 +214,18 @@ static struct {
 	{"Sanyo", "VPC-G200", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
 	{"Sanyo", "VPC-G200EX", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
 	{"Sanyo", "VPC-G210", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Sanyo", "VPC-G250", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Sanyo", "VPC-G250", 	SIERRA_MODEL_DEFAULT,	0, 0, SIERRA_NO_51 },
 	{"Sierra Imaging", "SD640",SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	/*
+	 * Added by Sean Bruno on July 22, 2004 to support the Toshiba
+	 * cameras below.
+	 */
+	{"Toshiba", "PDR-M60", SIERRA_MODEL_DEFAULT, 0x1132, 0x4332,
+		SIERRA_NO_51 | SIERRA_NO_REGISTER_40 | SIERRA_NO_USB_CLEAR},
+	{"Toshiba", "PDR-M61", SIERRA_MODEL_DEFAULT, 0x1132, 0x4335,
+		SIERRA_NO_51 | SIERRA_NO_REGISTER_40 | SIERRA_NO_USB_CLEAR},
+	{"Toshiba", "PDR-M65", SIERRA_MODEL_DEFAULT, 0x1132, 0x4334,
+		SIERRA_NO_51 | SIERRA_NO_REGISTER_40 | SIERRA_NO_USB_CLEAR},
 	{NULL, NULL, 0,	0, 0, 0}
 };
 
@@ -216,12 +266,22 @@ int camera_abilities (CameraAbilitiesList *list)
 		a.speed[0] = 9600;
 		a.speed[1] = 19200;
 		a.speed[2] = 38400;
-		if (!(sierra_cameras[x].flags & SIERRA_LOW_SPEED)) {
-			a.speed[3] = 57600;
-			a.speed[4] = 115200;
-			a.speed[5] = 0;
-		} else {
+		if (sierra_cameras[x].flags & SIERRA_LOW_SPEED) {
+			/*
+			 * Camera supports 9600 -> 38400 
+			 */
 			a.speed[3] = 0;
+		} else {
+			a.speed[3] = 57600;
+			if (sierra_cameras[x].flags & SIERRA_MID_SPEED) {
+				/*
+				 * Camera supports 9600 -> 57600.
+				 */
+				a.speed[4] = 0;
+			} else {
+				a.speed[4] = 115200;
+				a.speed[5] = 0;
+			}
 		}
 		a.operations        = 	GP_OPERATION_CAPTURE_IMAGE |
 					GP_OPERATION_CAPTURE_PREVIEW |
@@ -318,6 +378,12 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		info->audio.fields |= GP_FILE_INFO_TYPE;
 	}
 
+#if 0 /* need to add GMT -> local conversion first */
+	if (i.date) {
+		info->file.fields |= GP_FILE_INFO_MTIME;
+		info->file.mtime = i.date; /* FIXME: convert from GMT to localtime */
+	}
+#endif
 	/* Type of image and preview? */
 	if (strstr (filename, ".MOV") != NULL) {
 		strcpy (info->file.type, GP_MIME_QUICKTIME);
@@ -615,14 +681,12 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 	GP_DEBUG ("*** folder: %s", folder);
 	GP_DEBUG ("*** filename: %s", filename);
 
-	id = gp_context_progress_start (context, 4, filename);
+	id = gp_context_progress_start (context, 4, "%s", filename);
 	/*
 	 * XXX the CHECK() macro's don't allow for cleanup on error.
 	 */
 	/* Get the file number from the CameraFilesystem */
-#ifdef GTKAM_IS_MODIFIED
 	gp_context_progress_update (context, id, 0);
-#endif
 	CHECK (n = gp_filesystem_number (camera->fs, folder, filename, context));
 	/* Set the working folder and delete the file */
 	CHECK (camera_start (camera, context));
@@ -1870,8 +1934,13 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *c)
 		sprintf (buf, _("%sSoftware Rev.: %s\n"), buf, t);
 
 	/* Get all the integer information */
-	if (sierra_get_int_register(camera, 40, &v, c) >= 0)
-		sprintf (buf, _("%sFrames Taken: %i\n"), buf, v);
+	if (camera->pl->flags & SIERRA_NO_REGISTER_40) {
+		if (sierra_get_int_register(camera, 10, &v, c) >= 0)
+			sprintf (buf, _("%sFrames Taken: %i\n"), buf, v);
+	} else {
+		if (sierra_get_int_register(camera, 40, &v, c) >= 0)
+			sprintf (buf, _("%sFrames Taken: %i\n"), buf, v);
+	}
 	if (sierra_get_int_register(camera, 11, &v, c) >= 0)
 		sprintf (buf, _("%sFrames Left: %i\n"), buf, v);
 	if (sierra_get_int_register(camera, 16, &v, c) >= 0)
@@ -1930,7 +1999,7 @@ camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 		    "    camera control menu appears. Set it to ON.\n"
 		    "(3) If you switch the 'LCD mode' to 'Monitor' or\n"
 		    "    'Normal', don't forget to switch it back to 'Off'\n"
-		    "    before disconnectig. Otherwise you cannot use\n"
+		    "    before disconnecting. Otherwise you cannot use\n"
 		    "    the camera's buttons. If you end up with this\n"
 		    "    state, you should reconnect the camera to the\n"
 		    "    PC and switch LCD to 'Off'."));
@@ -1949,7 +2018,7 @@ camera_about (Camera *camera, CameraText *about, GPContext *context)
 		_("sierra SPARClite library\n"
 		"Scott Fritzinger <scottf@unr.edu>\n"
 		"Support for sierra-based digital cameras\n"
-		"including Olympus, Nikon, Epson, and others.\n"
+		"including Olympus, Nikon, Epson, Pentax, and others.\n"
 		"\n"
 		"Thanks to Data Engines (www.dataengines.com)\n"
 		"for the use of their Olympus C-3030Z for USB\n"
@@ -2000,6 +2069,17 @@ int get_jpeg_data(const char *data, int data_size, char **jpeg_data, int *jpeg_s
 	return ret_status;
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.folder_list_func = folder_list_func,
+	.get_info_func = get_info_func,
+	.set_info_func = set_info_func,
+	.get_file_func = get_file_func,
+	.put_file_func = put_file_func,
+	.del_file_func = delete_file_func,
+	.delete_all_func = delete_all_func,
+};
+
 int
 camera_init (Camera *camera, GPContext *context) 
 {
@@ -2156,14 +2236,7 @@ camera_init (Camera *camera, GPContext *context)
 
         /* We start off not knowing where we are */
         strcpy (camera->pl->folder, "");
-        CHECK_STOP_FREE (camera, gp_filesystem_set_list_funcs (camera->fs,
-				file_list_func, folder_list_func, camera));
-        CHECK_STOP_FREE (camera, gp_filesystem_set_info_funcs (camera->fs,
-                                get_info_func, set_info_func, camera));
-	CHECK_STOP_FREE (camera, gp_filesystem_set_file_funcs (camera->fs,
-				get_file_func, delete_file_func, camera));
-	CHECK_STOP_FREE (camera, gp_filesystem_set_folder_funcs (camera->fs,
-				put_file_func, delete_all_func, NULL, NULL, camera));
+        CHECK_STOP_FREE (camera, gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera));
 
         CHECK (camera_stop (camera, context));
 
