@@ -28,7 +28,7 @@
  */
 
 #include "config.h"
-#include <gphoto2-port-library.h>
+#include <gphoto2/gphoto2-port-library.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -73,9 +73,9 @@
 #  include <lockdev.h>
 #endif
 
-#include <gphoto2-port-result.h>
-#include <gphoto2-port-log.h>
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2-port-result.h>
+#include <gphoto2/gphoto2-port-log.h>
+#include <gphoto2/gphoto2-port.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -99,7 +99,7 @@
 #define CHECK(result) {int r=(result); if (r<0) return (r);}
 
 /* Linux */
-#if defined(__linux__)
+#ifdef __linux__
 /* devfs is accounted for in the implementation */
 #define GP_PORT_SERIAL_PREFIX   "/dev/ttyS%i"
 #define GP_PORT_SERIAL_RANGE_LOW        0
@@ -107,36 +107,40 @@
 #endif
 
 /* FreeBSD */
-#if defined(__FreeBSD__)
+#ifdef __FreeBSD__
+#if __FreeBSD_version < 600000
 #define GP_PORT_SERIAL_PREFIX   "/dev/cuaa%x"
+#else
+#define GP_PORT_SERIAL_PREFIX   "/dev/cuad%x"
+#endif
 #define GP_PORT_SERIAL_RANGE_LOW        0
 #define GP_PORT_SERIAL_RANGE_HIGH       (0xf)
 #endif
 
 /* OpenBSD */
 /* devices appear to go up to /dev/cuac7, but we just list the first 4 */
-#if defined(__OpenBSD__)
+#ifdef __OpenBSD__
 # define GP_PORT_SERIAL_PREFIX   "/dev/cua%02x"
 # define GP_PORT_SERIAL_RANGE_LOW        0
 # define GP_PORT_SERIAL_RANGE_HIGH       3
 #endif
 
 /* NetBSD */
-#if defined(__NetBSD__)
+#ifdef __NetBSD__
 #define GP_PORT_SERIAL_PREFIX   "/dev/tty0%i"
 #define GP_PORT_SERIAL_RANGE_LOW        0
 #define GP_PORT_SERIAL_RANGE_HIGH       32
 #endif
 
 /* Tru64 UNIX */
-#if defined(__osf__)
+#ifdef __osf__
 #define GP_PORT_SERIAL_PREFIX   "/dev/tty%02i"
 #define GP_PORT_SERIAL_RANGE_LOW        0
 #define GP_PORT_SERIAL_RANGE_HIGH       4
 #endif
 
 /* Darwin */
-#if defined(__APPLE__)
+#ifdef __APPLE__
 /* This is the Keyspan USB serial adapter device (UNTESTED) */
 #define GP_PORT_SERIAL_PREFIX   "/dev/tty.KeyUSA28X%i"
 #define GP_PORT_SERIAL_RANGE_LOW        111
@@ -303,9 +307,7 @@ gp_port_library_list (GPPortInfoList *list)
 	GPPortInfo info;
 	char path[1024], prefix[1024];
         int x, fd;
-#ifdef __linux
         struct stat s;
-#endif
 #ifdef OS2
         int r, fh, option;
 #endif
@@ -331,6 +333,10 @@ gp_port_library_list (GPPortInfoList *list)
 		if (r)
 			continue;
 #endif
+		/* Very first of all, if the device node is not there,
+		 * there is no need to try locking. */
+		if ((stat (path, &s) == -1) && ((errno == ENOENT) || (errno == ENODEV)))
+			continue;
 
 		/* First of all, try to lock the device */
 		if (gp_port_serial_lock (NULL, path) < 0)
@@ -360,7 +366,7 @@ gp_port_library_list (GPPortInfoList *list)
 		gp_port_serial_unlock (NULL, path);
 		info.type = GP_PORT_SERIAL;
 		strncpy (info.path, "serial:", sizeof (info.path));
-		strncat (info.path, path, sizeof (info.path));
+		strncat (info.path, path, sizeof (info.path) - strlen (info.path) - 1);
 		snprintf (info.name, sizeof (info.name),
 			  _("Serial Port %i"), x);
 		CHECK (gp_port_info_list_append (list, info));
@@ -438,7 +444,7 @@ gp_port_serial_open (GPPort *dev)
 	}
 	dev->pl->fd = -1;
 
-#if defined(__FreeBSD__) || defined(__APPLE__)
+#if defined(__FreeBSD__) || defined(__APPLE__) || defined(__DragonFly__)
         dev->pl->fd = open (port, O_RDWR | O_NOCTTY | O_NONBLOCK);
 #elif OS2
         fd = open (port, O_RDWR | O_BINARY);
@@ -845,12 +851,10 @@ gp_port_serial_check_speed (GPPort *dev)
         tio.c_cflag = (tio.c_cflag & ~CSIZE) | CS8;
 
         /* Set into raw, no echo mode */
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
         tio.c_iflag &= ~(IGNBRK | IGNCR | INLCR | ICRNL |
                          IXANY | IXON | IXOFF | INPCK | ISTRIP);
-#else
-        tio.c_iflag &= ~(IGNBRK | IGNCR | INLCR | ICRNL | IUCLC |
-                         IXANY | IXON | IXOFF | INPCK | ISTRIP);
+#ifdef IUCLC
+        tio.c_iflag &= ~IUCLC;
 #endif
         tio.c_iflag |= (BRKINT | IGNPAR);
         tio.c_oflag &= ~OPOST;
